from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains, NotEquals

from autopilot.input import Mouse, Touch, Pointer

from time import sleep

import os
 

 

class EvinceTests(AutopilotTestCase):
 
    def setUp(self):
        super(EvinceTests, self).setUp()
        self.app = self.launch_test_application('evince')

        self.windowTitle = self.app.select_single('EvWindow') 
        self.prev_button = self.app.select_single('GtkButton', tooltip_text='Go to the previous page')
        self.next_button = self.app.select_single('GtkButton', tooltip_text='Go to the next page')
        self.page_num_box = self.app.select_single('GtkBox', tooltip_text='Select Page')
        self.search_button = self.app.select_single('GtkToggleButton',
                                                    tooltip_text='Find a word or phrase in the document')
        self.entry = self.page_num_box.get_children_by_type('GtkEntry')[0]
        self.pointing_device = Pointer(Mouse.create())
        
    def test_display_pdf_file(self):
        '''test to view a pdf file'''
    
        #open the created temp file
        self.open_temp_file()
        #Grab evince window       
        
        #test file name is in the window title bar
        self.assertThat(self.windowTitle.title, Eventually(Contains('EvinceTestPdf.pdf')))
        # delete the temp file
                
    def test_changing_pages(self):
        #Lets test we can go up and down the pages
        
        self.open_temp_file()
        self.assertThat(self.windowTitle.title, Eventually(Contains('EvinceTestPdf.pdf')))
        # move to page 2
        self.pointing_device.move_to_object(self.next_button)
        self.pointing_device.click()
        self.assertThat(self.entry.text, Eventually(Contains('2')))
        
        
        self.pointing_device.move_to_object(self.prev_button)
        self.pointing_device.click()
        
        self.assertThat(self.entry.text, Eventually(Contains('1')))
        
        
        
    def test_sidebar_open_close(self):
        #Lets test we can open and close the sidebar
        self.open_temp_file()
        self.assertThat(self.windowTitle.title, Eventually(Contains('EvinceTestPdf.pdf')))
        # test opening and closing of sidebar
        self.change_sidebar_state()
        self.change_sidebar_state()
                
        
    
    def test_search_document(self):
        #lets test we can search a document
        self.open_temp_file()
        self.assertThat(self.windowTitle.title, Eventually(Contains('EvinceTestPdf.pdf')))
        #click search toggle button
        self.pointing_device.move_to_object(self.search_button)
        self.pointing_device.click()
        # Enter a search query
        self.keyboard.type('Welcome')
        # check it found the the search query
        self.search_result_label = self.app.select_single('GtkLabel', label='1 found on this page')
        self.assertThat(self.search_result_label.label, Eventually(Contains('1 found on this page')))
        
        
 
    def change_sidebar_state(self):
        ''' changes the state of the sidebar and tests that it changed '''
        # get value of Evsidebar.visible
        self.side_bar = self.app.select_single('EvSidebar')
        visible = self.side_bar.visible
        # Test and change the side bar, 
        if visible == 1:
            self.keyboard.press_and_release('F9')
            self.assertThat(self.side_bar.visible, Equals(0))
        else:
            self.keyboard.press_and_release('F9')
            self.assertThat(self.side_bar.visible, Equals(1))
    
    def open_temp_file(self):
        '''Opens the EvinceTestPdf.pdf file'''
        # Click on the settings button
        self.settings_button = self.app.select_single('GtkImage', icon_name='emblem-system-symbolic')
        self.pointing_device.move_to_object(self.settings_button)
        self.pointing_device.click()
        # press 'O' to open the file dialog
        self.keyboard.press_and_release("o")
        # check the file dialog opened and contains 'Open Document' in the window title
        self.open_file_dialog = self.app.select_single('GtkFileChooserDialog')
        self.assertThat(self.open_file_dialog.title, Eventually(Equals('Open Document')))
        # enter the file name and location
        self.keyboard.press_and_release("Ctrl+a")
        self.keyboard.type('/')
        self.keyboard.press_and_release('Enter')
        self.keyboard.press_and_release('Ctrl+a')
        testFile = 'EvinceTestPdf.pdf'
        for root, dirs, files in os.walk(os.getcwd()):
            for name in files:
                if name == testFile:
                    test_pdf = os.path.abspath(os.path.join(root, name))
                    self.keyboard.type(test_pdf)
        # click on the 'open' button
        self.keyboard.press_and_release('Enter')
        
    

    
    
    

