#Note: This testcase avoids using introspection because autopilot-gtk fails to introspect firefox properly
from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains
from autopilot.process import ProcessManager

#register firefox as an application so we can call it
ProcessManager.register_known_application("Firefox", "firefox.desktop", "firefox")

class FirefoxTests(AutopilotTestCase):

    def setUp(self):
        super(FirefoxTests, self).setUp()
        self.manager = ProcessManager.create('BAMF')
        self.app = self.manager.start_app_window("Firefox")

        #make sure firefox is up and loaded
        self.app.set_focus()
        self.assertTrue(self.app.is_focused)

    def test_browse_planet_ubuntu(self):
        #Test-case name: firefox/fir-001
        #This test will check that Firefox can display webpages

        #open location dialog
        self.keyboard.press_and_release("Ctrl+l")

        #type in planet ubuntu
        self.keyboard.type("http://planet.ubuntu.com\n")

        #check that page title loaded planet ubuntu
        self.assertThat(lambda: self.app.name, Eventually(Contains("Planet Ubuntu"), timeout=30))

    def test_google_search_suggestions(self):
        #Test-case name: firefox/fir-002
        #This test will check that Firefox can display search suggestions

        #open search box, switch to google
        self.keyboard.press_and_release("Ctrl+k")
        self.keyboard.press_and_release("F4")
        self.keyboard.press_and_release("g")

        #search for ubuntu server docs
        self.keyboard.type("ubuntu server docs")

        #ensure we have a generated list of suggestions
        #WE CAN'T DO THIS :-(

        #go to results page
        self.keyboard.type("\n")

        #check results by looking at page title
        self.assertThat(lambda: self.app.name, Eventually(Contains("Google")))
        
        self.assertThat(lambda: self.app.name, Eventually(Contains("server docs")))

    def test_search_jenkins(self):
        
        #This test will check that Firefox can add and use different search engines
        self.keyboard.press_and_release('Ctrl+l')
        
        self.keyboard.type('http://jenkins.qa.ubuntu.com')
        
        self.keyboard.press_and_release('Enter')
        
        self.assertThat(lambda: self.app.name, Eventually(Contains('Jenkins'), timeout=30))
        #open search box, add jenkins to search engine list
        self.keyboard.press_and_release("Ctrl+k")
        self.keyboard.press_and_release("F4")
        self.keyboard.press_and_release('Up')
        self.keyboard.press_and_release('Up')
        self.keyboard.press_and_release('Enter')
        

        #search for 'saucy'
        self.keyboard.type("saucy \n")

        #check that page title loaded saucy on jenkins
        self.assertThat(lambda: self.app.name, Eventually(Contains("saucy")))
        self.assertThat(lambda: self.app.name, Eventually(Contains("Jenkins")))

    def test_tabs_and_windows(self):
        #This test will check that Firefox can create tabs and windows

        #test fullscreen
        self.keyboard.press_and_release("F11")

        #verify fullscreen
        #Does Evince enter fullscreen mode?
        self.assertThat(lambda: self.app._get_window_states(), Eventually(Contains("_NET_WM_STATE_FULLSCREEN")))

        #go back to normal
        self.keyboard.press_and_release("F11")

        #open new tab
        self.keyboard.press_and_release("Ctrl+t")

        #check that page title reflects new tab
        self.assertThat(lambda: self.app.name, Eventually(Equals("Mozilla Firefox")))

        #close new tab
        self.keyboard.press_and_release("Ctrl+w")

        #open new window
        originalWindowCount = len(self.manager.get_open_windows_by_application("Firefox"))
        self.keyboard.press_and_release("Ctrl+n")

        #check that we spawned a new window; we use window count so this works even if you have firefox already running (shouldn't happen in a sterile labe enviroment however)
        self.assertThat(lambda: len(self.manager.get_open_windows_by_application("Firefox")), Eventually(Equals(originalWindowCount + 1)))

        #close window
        self.keyboard.press_and_release("Ctrl+w")
