from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Contains
from autopilot.process import ProcessManager

from time import sleep
import tempfile
import os


class GeditTests(AutopilotTestCase):
    
    def setUp(self):
        super(GeditTests, self).setUp()
        self.app = self.launch_test_application("gedit")

    def test_save_and_open_a_file(self):
        #Test-case name: gedit/ged-001
        #This test will check that GEdit can save and open saved files
        #close the original blank document
        self.keyboard.press_and_release("Ctrl+w")

        #documents = self.app.select_single('GeditTab')
        #self.assertThat(lambda: self.app.select_single('GeditTab').name, Eventually(Contains(fileName)))

        documents = self.app.select_many('GeditTab')
        self.assertThat(lambda: len(documents), Eventually(Equals(0)))

        #open a new blank document
        btn = self.app.select_single('GtkToolButton', label='_New')
        self.assertThat(btn, NotEquals(None))
        self.mouse.move_to_object(btn)
        self.mouse.click()

        #hacky delay to wait
        sleep(1)

        self.keyboard.type("The quick brown foxes jump over the lazy dog")

        #save file
        tempFile = _generateTempFileName()
        self._saveFile(tempFile)

        #Close the open file
        self.keyboard.press_and_release("Ctrl+w")

        #Make sure it closed
        documents = self.app.select_many('GeditTab')
        self.assertThat(lambda: len(documents), Eventually(Equals(0)))

        #Open the file we made
        btn = self.app.select_single('GeditOpenToolButton', label='Open')
        self.assertThat(btn, NotEquals(None))
        self.mouse.move_to_object(btn)
        self.mouse.click()

        #hacky delay wait for window to appear :-(
        sleep(3)

        #Manually type location
        self.keyboard.type(tempFile + "\n")

        #Check to see that the title reflects our filename
        #(dirName, fileName) = os.path.split(tempFile)
        #documents = self.app.select_single('GeditTab')
        #self.assertThat(lambda: documents[0].name, Eventually(Contains(fileName)))

        #Check that the file contains the phrase "The quick brown foxes jump over the lazy dog"
        filePhrase = open(tempFile, 'r')
        self.assertThat(filePhrase, Contains("The quick brown foxes jump over the lazy dog\n"))

        #Close the open file
        self.keyboard.press_and_release("Ctrl+w")

        #Delete the file we made
        #Added sleep so file does not get removed too soon
        sleep(1)
        self.addCleanup(os.remove, tempFile)

    def test_replace_test(self):
        #Test-case name: gedit/ged-003
        #This test will check that GEdit can replace text

        #Type in phrase
        self.keyboard.type("The quick brown foxes jump over the lazy dog")

        #Press ctrl+h to open replace dialog
        self.keyboard.press_and_release("Ctrl+h")

        #Verify replace dialog appears
        #TODO

        #Replace foxes with fox
        self.keyboard.type("foxes\t\tfox")
        self.keyboard.press_and_release("Alt+a")

        #Verify replace dialog stays open
        #TODO

        #Close replace dialog
        self.keyboard.press_and_release("Escape")

        #Save file
        tempFile = _generateTempFileName()
        print "tempFile " + tempFile
        self._saveFile(tempFile)

        #Verify text now reads 'The quick brown fox jump over the lazy dog'
        savedFile = open(tempFile, "r")
        filePhrase = savedFile.read()
        self.assertThat(filePhrase, Contains("The quick brown fox jump over the lazy dog\n"))

        #Close the open file
        self.keyboard.press_and_release("Ctrl+w")

        #Delete the file we made
        sleep(3)
        self.addCleanup(os.remove, tempFile)

    def test_recent_documents(self):
        #GeditOpenToolButton tooltip_text Open '/tmp/'
        recentBtn = self.app.select_single('GeditOpenToolButton', label='Open')
        self.assertThat(recentBtn, NotEquals(None))
        self.mouse.move_to_object(recentBtn)
        self.mouse.click()
        #inserted sleep to see the recent files otherwise window closes too quicly
        sleep(2)
        
    def test_highlight_searched_text(self):
        #Test-case name: gedit/ged-004
        #This test will check that GEdit can find and highlight searched text

        #Type in phrase
        self.keyboard.type("The quick brown foxes jump over the lazy dog")

        #Press Ctrl+f to open search box
        self.keyboard.press_and_release("Ctrl+f")

        #Verify search box appears
        #TODO

        #Type in "T"
        self.keyboard.type("T")
        sleep(3)

        #Verify all the "T" letters in the document are highlighted
        #TODO

        #Type in "he"
        self.keyboard.type("he")

        #Verify all the "The" words in the document are highlighted
        #TODO

        #Press Ctrl+g to go to the next searched word
        self.keyboard.press_and_release("Ctrl+g")

        #Verify that text cursor moves to grey-highlighted 'the' in 'over the lazy dog'
        #TODO

        #Press Shift+Ctrl+g to go to the previous searched word
        self.keyboard.press_and_release("Shift+Ctrl+g")

        #Verify that text cursor moves to grey-highlighted 'The' in 'The quick brown'
        #TODO

        #Close search box
        self.keyboard.press_and_release("Escape")

        #Verify that the seach box is closed
        #TODO

        #Verify all the "The" words in the document are highlighted
        #TODO

        #Close the open file
        self.keyboard.press_and_release("Ctrl+w")

        #Don't save
        sleep(3)
        self.keyboard.press_and_release("Alt+w")

    def test_display_file_browser_panel(self):
        #Test-case name: gedit/ged-005
        #This test will check that GEdit can display File Browser panel

        #These property's exist
        #GeditCloseButton tooltip_text "Hide panel"
        #GtkPaned GtkBox GtkBox GtkBox GtkLabel label "Documents"
        #GtkPaned GtkBox GtkBox GtkBox GtkLabel label "File Browser"
        #GtkFileBrowserWidget
        #GeditDocumentsPanel
        #GtkWindow GtkMenu GtkLabel label "Documents"
        #GtkWindow GtkMenu GtkLabel label "File Browser"

        #Press Alt+e to open "Edit" menu
        self.keyboard.press_and_release("Alt+e", delay=0.5)

        #Press "e" to open "Preferences"
        self.keyboard.press_and_release("e")

        #combo_box_widget = self.app.select_single('ComboBox', name='foobar', someOtherProperty=123)

        #Press left arrow  "Plugins" tab
        self.keyboard.press_and_release("Left")

        #Verify the "File Browser Panel" plugin is checked
        #TODO

        #Press Escape to close preferences window
        self.keyboard.press_and_release("Escape")

        #Press F9
        self.keyboard.press_and_release("F9")

        #Verify the The side panel appears with the file browser panel included
        #should spawn "Documents"
        #introspection
        #self.assertThat(self.app.select_many('GtkLabel', label='Documents'), NotEquals(None))

        #Press F9
        self.keyboard.press_and_release("F9")

        #Verify the The side panel close properly
        #this seems to stick around the tree -- look for visible property?
        #self.assertThat(self.app.select_many('GtkLabel', label='Documents'), Equals(None))
        #self.assertThat(self.app.select_many('GeditCloseButton', tooltip_text='Hide panel'), Equals(None))

    def test_tab_manipulation(self):
        return
        #test using alt keys to move between docus
        #test using ctrl+w to close tabs
        #test ctrl+t for new tabs
        #test button for new tab
        #test creating tab group
        #test moving tab group to new window

    #def test_insert_overwrite_mode(self):
        #verify we are in insert mode
        #GeditWindow GtkBox GeditStatusbar GtkLabel
        #typeMode = self.app.select_single('GtkLabel', label=' INS ')
        #typeMode = self.app.select_many('GtkLabel', label=' INS ')
        #self.assertThat(typeMode, NotEquals(None))


        #type out some words

        #turn on overwrite, and press home
        #self.keyboard.press_and_release("Home")
        #self.keyboard.press_and_release("Insert")
        #self.assertThat(typeMode.label, Equals(' OVR '))
        

        #type over the first word
        #turn on insert
        #add a word in the middle
        #save file and check result matches

    def test_shortcuts(self):
        return
        #test search, go to line
        #plugin! test edit, insert date and time
        #test copy and paste into doc
        #test save all
        #test close all
        #test revert
        #test undo and redo

    def test_toolbar(self):
        #test new
        #GeditWindow GtkBox GtkToolbar GtkToolButton
        #grab the file new button
        btn = self.app.select_single('GtkToolButton', label='_New')

        #does it exist?
        self.assertThat(btn, NotEquals(None))

        #let's click it to open a new document
        documents = self.app.select_many('GeditTab')
        orginalNumDocuments = len(documents)
        self.mouse.move_to_object(btn)
        self.mouse.click()

        #let's re-grab the open documents
        documents = self.app.select_many('GeditTab')

        #do we have another document now?
        self.assertThat(lambda: len(documents), Eventually(Equals(orginalNumDocuments + 1)))

        #is it a new untitled document?
        #This will grab the current window title, 'Untitled Document # - gedit', but the untitled document is translated. For now, we'll ignore the translatable text to ensure compatibility
        documents = self.app.select_many('GeditTab')
        #note index starts at 0
        self.assertThat(documents[orginalNumDocuments].name, Contains(str(orginalNumDocuments + 1)))

        #test open
        btn = self.app.select_single('GeditOpenToolButton', label='Open')
        self.assertThat(btn, NotEquals(None))

        #test save
        btn = self.app.select_single('GtkToolButton', label='Save')
        self.assertThat(btn, NotEquals(None))

        #test print
        btn = self.app.select_single('GtkToolButton', label='Print')
        self.assertThat(btn, NotEquals(None))

        #test undo
        btn = self.app.select_single('GtkToolButton', label='_Undo')
        self.assertThat(btn, NotEquals(None))

        #test redo
        btn = self.app.select_single('GtkToolButton', label='_Redo')
        self.assertThat(btn, NotEquals(None))

        #test cut
        btn = self.app.select_single('GtkToolButton', label='Cu_t')
        self.assertThat(btn, NotEquals(None))

        #test copy
        btn = self.app.select_single('GtkToolButton', label='_Copy')
        self.assertThat(btn, NotEquals(None))

        #test paste
        btn = self.app.select_single('GtkToolButton', label='_Paste')
        self.assertThat(btn, NotEquals(None))

        #test search
        btn = self.app.select_single('GtkToolButton', label='Find')
        self.assertThat(btn, NotEquals(None))

        #test replace
        btn = self.app.select_single('GtkToolButton', label='Replace')
        self.assertThat(btn, NotEquals(None))

    def _saveFile(self, saveName):
        saveBtn = self.app.select_single('GtkToolButton', label='Save')
        self.assertThat(saveBtn, NotEquals(None))
        self.mouse.move_to_object(saveBtn)
        self.mouse.click()

        #hacky delay wait for window to appear :-(
        sleep(3)

        self.keyboard.type(saveName)
        self.keyboard.press_and_release("Alt+s")

        #Check to see that the title reflects our filename
        (dirName, fileName) = os.path.split(saveName)
        documents = self.app.select_many('GeditTab')
        self.assertThat(documents[len(documents) - 1].name, Eventually(Contains(fileName)))

        #An asterisk in a gedit title means an unsaved document
        self.assertNotIn(documents[len(documents) - 1].name, "*")

def _generateTempFileName():
    #Create temporary file, then close it, so we can re-use the file name
    sFile = tempfile.NamedTemporaryFile()
    sFile.close()
    return sFile.name
