from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains
from autopilot.process import ProcessManager
import tempfile
import os
from time import sleep

#Register Nautilus as an application so we can call it
ProcessManager.register_known_application("File Browser", "nautilus.desktop", "nautilus")

#http://packages.qa.ubuntu.com/qatracker/testcases/1424/info
class NautilusTests(AutopilotTestCase):

    def setUp(self):
        super(NautilusTests, self).setUp()
        self.proc_manager = ProcessManager.create('BAMF')
        self.app = self.proc_manager.start_app_window("File Browser")

        #This will make sure nautilus window you launched is focused and ready to accept your keyboard input
        self.app.set_focus()
        #We need to assert the window title = Home here to keep focus of nautilus. #
        #It fails with assertTrue(self.app.is_focused)
        self.assertThat(lambda: self.app.title, Eventually(Contains('Home')))


    def test_create_a_folder(self):
        #Test-case name: nautilus/TC-NFM-002
        #This test will check that user can create folders in Nautilus

        #Select tmp folder
        self.keyboard.press_and_release("Ctrl+l")
        self.keyboard.type('/tmp/')
        self.keyboard.press_and_release('Enter')
        #make new folder
        tempName=_generateTempFileName()
        (dirName, fileName) = os.path.split(tempName)
        self.keyboard.press_and_release("Ctrl+Shift+n")
        #use the generated filename as folder name in this case
        self.keyboard.type(fileName)
        self.keyboard.press_and_release('Enter')
        self.keyboard.press_and_release('Enter')
        #verify folder
        self.assertThat(lambda: self.app.title, Eventually(Contains(fileName)))

        #delete folder
        self.addCleanup(os.rmdir, tempName)

def _generateTempFileName():
    #Create temporary file, then close it, so we can re-use the file name
    sFile = tempfile.NamedTemporaryFile()
    sFile.close()
    return sFile.name
