from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains, NotEquals
from autopilot.process import ProcessManager
from autopilot.input import Mouse, Pointer

from time import sleep
import tempfile
import os
import shutil

#http://packages.qa.ubuntu.com/qatracker/testcases/1421/info
class ScreenshotTests(AutopilotTestCase):

    def setUp(self):
        super(ScreenshotTests, self).setUp()
        self.app = self.launch_test_application("gnome-screenshot", "-i")

        # only instantiate these once for multiple tests
        self.pointing_device = Pointer(Mouse.create())
        self.archiveWindow = self.app.select_single('GtkDialog')

        #verify window title: Take Screenshot to verify Screenshot is correclty opened
        self.assertThat(self.archiveWindow.title, Eventually(Contains('Take Screenshot')))
        
        self.take_screenshot_button = self.app.select_single('GtkLabel', label='Take _Screenshot')
        self.grab_whole_screen_rb = self.app.select_single('GtkRadioButton', label='Grab the whole sc_reen')
        self.grab_curr_window_rb = self.app.select_single('GtkRadioButton', label='Grab the current _window')
        self.select_grab_area_rb=self.app.select_single('GtkRadioButton', label='Select _area to grab')
        self.incl_win_border=self.app.select_single('GtkCheckButton', label='Include the window _border')
##        self.drop_shadow_effect_shadow=self.app.select_single('GtkComboBoxAccessible', accessible_name='shadow')
##        self.drop_shadow_effect_none=self.app.select_single('GtkComboBoxAccessible', accessible_name='none')
##        self.drop_shadow_effect_border=self.app.select_single('GtkComboBoxAccessible', accessible_name='border')
    
        #potentially useful objects
        #self.border_effect=self.app.select_single('GtkComboBoxAccessible', accessible_name='border')
        #self.no_effect=self.app.select_single('GtkComboBoxAccessible', accessible_name='none')
        #self.app.select_single('GtkLabel', label='Grab after a _delay of')
        #self.app.select_single('GtkCheckButton', label='Include _pointer')
        #self.app.select_single('GtkToggleButtonAccessible', accessible_name='Include pointer')
        #self.app.select_single('GtkToggleButtonAccessible', accessible_name='Include the window border')
        #self.app.select_single('GtkMenuItemAccessible', accessible_name='Drop shadow')
        #self.app.select_single('GtkLabel', label='Apply _effect:')
        #self.app.select_single('GtkLabelAccessible', accessible_name='Apply effect:')        
        
    def test_whole_screen(self):
        #Test-case name: gnomescreenshot/gns-001
        #This test will check that Gnome Screenshot can save screenshot of the whole screen

        #Grab the whole desktop
        #Select Grab Whole Screen radio button and click on it
        self.pointing_device.move_to_object(self.grab_whole_screen_rb)
        self.pointing_device.click()

        #Select Take screenshot button and click it
        self.click_take_screenshot()

        #verify take screenshot window opens
        tempArchive = _tempArchiveName()
        (dirName, fileName) = os.path.split(tempArchive)
        self.save_screenshot_file(fileName)

        #Is the screenshot image saved to the /tmp directory? Is it a valid image file containing your screen?
        #verify it's a non-zero file
        self.verify_non_zero_file(fileName)
        
        #Cleanup
        self.delete_temp_file(tempArchive)

    def test_selected_window(self):
        #Test-case name: gnomescreenshot/gns-002
        #This test will check that Gnome Screenshot can save screenshot of selected window

        #Select Grab the current window
        self.pointing_device.move_to_object(self.grab_curr_window_rb)
        self.pointing_device.click()
        #Select Take screenshot button and click it
        self.click_take_screenshot()

        #verify take screenshot window opens
        tempArchive = _tempArchiveName()
        (dirName, fileName) = os.path.split(tempArchive)
        self.save_screenshot_file(fileName)

        #Is the screenshot image saved to the /tmp directory? Is it a valid image file containing your screen?
        #verify it's a non-zero file
        self.verify_non_zero_file(fileName)
        
        #Cleanup
        self.delete_temp_file(tempArchive)

    def test_drop_shadow_effect(self):
        #Test-case name: gnomescreenshot/gns-003
        #This test will check that Gnome Screenshot can save screenshot of selected window
        
        #Select Grab the current window
        self.pointing_device.move_to_object(self.grab_curr_window_rb)
        self.pointing_device.click()
        
        #Verify the "Include the window border" box is selected
        if self.incl_win_border.active == 0:
            self.pointing_device.move_to_object(self.incl_win_border)
            self.pointing_device.click()
        self.assertThat(lambda: self.incl_win_border.active, Eventually(Equals(1)))

##        #Select Drop shadow from the Apply effect drop down menu
##        if self.drop_shadow_effect_none.enabled == 'True':
##           self.pointing_device.move_to_object(self.drop_shadow_effect_none)
##        elif self.drop_shadow_effect_shadow.enabled == 'True':
##           self.pointing_device.move_to_object(self.drop_shadow_effect_shadow)    
##        else:
##           self.pointing_device.move_to_object(self.drop_shadow_effect_border)
##        self.pointing_device.click()
##        self.keyboard.press_and_release('Down')
##        self.keyboard.type('\n')
##        self.pointing_device.click()
##        self.pointing_device.click()
        
##       #Verify the "Drop shadow"  is selected
###       don't use these assertions
###        self.assertThat(lambda: self.drop_shadow_effect.selected, Eventually(Equals(True)))
###        self.assertThat(lambda: self.drop_shadow_effect.focused, Eventually(Equals(True)))
##        self.assertThat(lambda: self.drop_shadow_effect.enabled, Eventually(Equals(True)))
    
        #Select Take screenshot button and click it
        self.click_take_screenshot()
            
        #Make sure screenshot window you launched is focused and ready to accept your keyboard input
        tempArchive = _tempArchiveName()
        (dirName, fileName) = os.path.split(tempArchive)
        self.save_screenshot_file(fileName)

       #Is the screenshot image saved to the /tmp directory? Is it a valid image file containing your screen?
        #verify it's a non-zero file
        self.verify_non_zero_file(fileName)

        #Cleanup
        self.delete_temp_file(tempArchive)

    def save_screenshot_file(self, fileName):
        check_fn = lambda: self.app.select_single('GtkDialog', title='Save Screenshot')
        self.assertThat(check_fn, Eventually(NotEquals(None)))
        #select everything so to cancel old file name
        self.keyboard.press_and_release('Ctrl+a')
        self.keyboard.type(fileName)
        self.keyboard.type('.png')
        #select /tmp folder
        self.save_in_folder_button=self.app.select_single('GtkFileChooserButton', title='Select a folder')
        self.pointing_device.move_to_object(self.save_in_folder_button)
        self.pointing_device.click()
        #get to "other"
        self.keyboard.press_and_release('End')
        #save in the tmp folder
        self.keyboard.type("\n/tmp\n")
        #Save screenshot
        self.save_button=self.app.select_single('GtkLabel', label='_Save')
        self.pointing_device.move_to_object(self.save_button)
        self.pointing_device.click()
        
    def verify_non_zero_file(self,fileName):
        self.assertThat(lambda: os.path.isfile('/tmp/'+fileName+'.png'), Eventually(Equals(True)))
        self.assertThat(lambda: os.stat('/tmp/'+ fileName+'.png').st_size, Eventually(NotEquals(0)))

    def delete_temp_file(self, tempArchive):
        #Delete the screenshot file
        self.addCleanup(os.unlink, tempArchive+'.png')

    def click_take_screenshot(self):
        #click on take screenshot button
        self.pointing_device.move_to_object(self.take_screenshot_button)
        self.pointing_device.click()        


#Create temporary file name
def _tempArchiveName():
    '''Temporary archive name.'''
    sFile = tempfile.NamedTemporaryFile()
    sFile.close()
    return sFile.name