from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, Contains
from autopilot.process import ProcessManager

import tempfile
import os

#http://packages.qa.ubuntu.com/qatracker/testcases/1422/info
class TerminalTests(AutopilotTestCase):

    def setUp(self):
        super(TerminalTests, self).setUp()
        self.proc_manager = ProcessManager.create('BAMF')
        self.app = self.proc_manager.start_app_window("Terminal")

        #Make sure terminal window you launched is focused and ready to accept your keyboard input
        self.app.set_focus()
        self.assertTrue(self.app.is_focused)


    def test_save_a_file(self):
        #Test-case name: gnometerminal/ter-001
        #This test will check that basic terminal commands work in Gnome Terminal

        #Type a command
        self.keyboard.type("touch /tmp/test-file\n")

        #Verify that test-file has been created


        #Enter ls -al
        self.keyboard.type("ls -al /tmp\n")

        #Verify output


        #Press up arrow
        self.keyboard.press_and_release("Up")

        #Verify output you ran the same command you ran previously

        #Delete the file we created
        self.addCleanup(os.remove, "/tmp/test-file")



    def test_change_colors(self):
        #Test-case name: gnometerminal/ter-002
        #This test will check that terminal colors and zoom level can be changed

        #Press Alt+e to open "Edit" menu
        self.keyboard.press_and_release("Alt+e", delay=0.5)

        #Press "o" to open "Profiles"
        self.keyboard.press_and_release("o")

        #Press right arrow  twice for  "colors" tab
        self.keyboard.press_and_release("Right")
        self.keyboard.press_and_release("Right")

        #Change built in themes
        self.keyboard.press_and_release("Tab")
        self.keyboard.type("\n", delay=1)
        self.keyboard.press_and_release("Tab")
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down", delay=0.3)

        #Verify colors have changed

        #Put configuration back as it was
        self.keyboard.press_and_release("Up")
        self.keyboard.press_and_release("Up")
        self.keyboard.press_and_release("Up")
        self.keyboard.press_and_release("Up")
        self.keyboard.press_and_release("Up")


        #Change text color
        self.keyboard.press_and_release("Tab")
        self.keyboard.type("\n")
        self.keyboard.type("\n")
        self.keyboard.press_and_release("Alt+s")

        #Verify text color has changed


        #Put text color back as it was
        self.keyboard.type("\n", delay=1)
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down")
        self.keyboard.press_and_release("Down")
        self.keyboard.type("\n")
        self.keyboard.press_and_release("Alt+s")
        self.keyboard.press_and_release("Shift+Tab")
        self.keyboard.press_and_release("Shift+Tab")
        self.keyboard.type("\n")

        #Set a Background Image
        self.keyboard.press_and_release("Shift+Tab")
        self.keyboard.press_and_release("Right")
        self.keyboard.press_and_release("Alt+b")
        self.keyboard.press_and_release("Alt+f")
        self.keyboard.press_and_release("Space")
        self.keyboard.type("/usr/share/backgrounds/Leftover_by_Sagar_Jain.jpg")
        self.keyboard.press_and_release("Alt+o")
        self.keyboard.press_and_release("Tab")
        self.keyboard.press_and_release("Tab")
        self.keyboard.press_and_release("Space")

        #Verify Terminal background changes


        #Put Terminal background back as it was
        self.keyboard.press_and_release("Alt+s")
        self.keyboard.press_and_release("Space", delay=0.2)

        #Close Profile preferences window
        self.keyboard.press_and_release("Alt+c")

        #Change the Zoom Level
        self.keyboard.press_and_release("Ctrl+plus")
        self.keyboard.press_and_release("Ctrl+plus")
        self.keyboard.press_and_release("Ctrl+plus")
        self.keyboard.press_and_release("Ctrl+plus")
        self.keyboard.press_and_release("Ctrl+-")
        self.keyboard.press_and_release("Ctrl+-")

        #Verify zoom level has changed


        #Put Zoom level back as it was
        self.keyboard.press_and_release("Ctrl+0")


    def test_terminal_coloring(self):
        #Test-case name: gnometerminal/ter-003
        #This test will check that terminal coloring is active

        #Type in touch /tmp/text1.txt
        self.keyboard.type("touch /tmp/text1.txt\n")

        #Type in ls
        self.keyboard.type("ls /tmp\n")

        #Verify txt is in black


        #Type in chmod 777 text1.txt
        self.keyboard.type("chmod 777 /tmp/text1.txt\n")

        #Type in ls
        self.keyboard.type("ls /tmp\n")


        #Verify txt should now be green

        #Delete the file we created
        self.addCleanup(os.remove, "/tmp/text1.txt")


    def test_launch_terminal(self):
        #Test-case name: gnometerminal/ter-004
        #This test will check that the terminal tab launches succesfully


        #Open Terminal using the dash, and typing 'terminal'
        self.keyboard.press_and_release("Super")
        self.keyboard.type("terminal\n")

        #Is an instance of gnome-terminal with the title '@: ~' opened, with the current directory set to the users home directory?


        #Close opened terminal
        self.keyboard.press_and_release("Ctrl+d", delay=0.2)

        #Press Ctrl+Alt+T
        self.keyboard.press_and_release("Ctrl+Alt+t")

        #Is an instance of gnome-terminal with the title '@: ~' opened, with the current directory set to the users home directory?


        #Close opened terminal
        self.keyboard.press_and_release("Ctrl+d")
        self.keyboard.press_and_release("Ctrl+d")
        #Had to put two Ctrl+d's as the first one closes the terminal that opens with the launch of this test


    def test_new_title(self):
        #Test-case name: gnometerminal/ter-005
        #This test will check that the terminal can be given a new title

        #From the main menu select 'Terminal > Set Title...'
        self.keyboard.press_and_release("Alt+t", delay=0.5)
        self.keyboard.press_and_release("s", delay=0.5)

        #Verify the Set Title dialog appears
        #Check to see that the title of the window is "Set Title"
        #self.assertThat(lambda: self.app.title, Eventually(Contains("Set Title")))
        

        #Enter 'Test Title' and click OK
        self.keyboard.type("Test Title")
        self.keyboard.press_and_release("Alt+o")

        #Is the title of the window now 'Test Title'?
        #Check to see that the title reflects our 'Test Title'
        self.assertThat(lambda: self.app.name, Eventually(Contains("Test Title")))

def _generateTempFileName():
    #Create temporary file, then close it, so we can re-use the file name
    sFile = tempfile.NamedTemporaryFile()
    sFile.close()
    return sFile.name



