from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, GreaterThan, LessThan, Contains
from autopilot.input import Mouse, Touch, Pointer

from time import sleep
import tempfile
import os



#http://packages.qa.ubuntu.com/qatracker/testcases/1429/info
class TotemTests(AutopilotTestCase):

    def setUp(self):
        #Open Totem
        super(TotemTests, self).setUp()
        self.app = self.launch_test_application('totem')
        
        
        self.windowTitle = self.app.select_single('GtkWindow', title='Videos')
        self.pointing_device = Pointer(Mouse.create())
        
        
    def test_play_and_pause_video(self):
        # check the window loaded
        self.assertThat(self.windowTitle.title, Eventually(Contains('Videos')))
        self.status_label = self.app.select_single('GtkLabel', label='Stopped')
        
        # add content to playlist and auto starts
        self.setup_video()
        ## TODO assert playing label
        
        self.assertThat(self.status_label.label, Eventually(Contains('Playing')))
        # press pause button
        ## currently not working
        #self.pause_button = self.app.select_single('GtkButton', tooltip_text='Pause')
        #self.pointing_device.move_to_object(self.pause_button)
        #self.pointing_device.click()
        self.keyboard.press_and_release('Ctrl+Space')
        
        self.assertThat(self.status_label.label, Eventually(Contains('Paused')))
        # check we can start it playing again
        self.keyboard.press_and_release('Ctrl+Space')
        self.assertThat(self.status_label.label, Eventually(Contains('Playing')))
        
    def setup_video(self, ):
        ''' select and add content to the playlist '''
        #get the sidebar button
        self.sidebar_button = self.app.select_single('GtkToggleButton', label='S_idebar')
        #make sure playlist window is displayed
        self.keyboard.press_and_release('Alt+v')
        self.keyboard.press_and_release('Up')
        self.keyboard.press_and_release('Up')
        self.keyboard.press_and_release('Enter')
        #check if sidebar is already open, if not then open it
        sideIsActive = self.sidebar_button.active
        if sideIsActive == 0:
            self.pointing_device.move_to_object(self.sidebar_button)
            self.pointing_device.click()
            sideIsActive = self.sidebar_button.active
        # check it is open
        self.assertThat(sideIsActive, Equals(1))
        
        #select the + button
        self.add_button = self.app.select_single('GtkToolButton', tooltip_text='Add...')
        #click on + button
        self.pointing_device.move_to_object(self.add_button)
        self.pointing_device.click()
        # check the file chooser dialog loaded
        self.file_chooser_dialog = self.app.select_single('GtkFileChooserDialog')
        self.assertThat(self.file_chooser_dialog.title, Eventually(Contains('Select Movies or Playlists')))
        # enter content path
        self.keyboard.press_and_release('Alt+d')
        self.keyboard.type('/usr/share/example-content/Ubuntu_Free_Culture_Showcase')
        # add the directory to the playlist
        self.add_dir_button = self.app.select_single('GtkLabel', label='Add Directory')
        self.pointing_device.move_to_object(self.add_dir_button)
        self.pointing_device.click()
        
        
    
        
        